@@ -4,7 +4,6 @@ import logging
 from math import ceil, floor
 import os
 import sys
-import warnings
 
 import click
 from cligj import (
@@ -19,11 +18,10 @@ from rasterio.rio.cli import cli, coords, write_features
 
 
 logger = logging.getLogger('rio')
-warnings.simplefilter('default')
 
 
 # Shapes command.
-@cli.command(short_help="Write the shapes of features.")
+@cli.command(short_help="Write shapes extracted from bands or masks.")
 @click.argument('input', type=click.Path(exists=True))
 @precision_opt
 @indent_opt
@@ -34,21 +32,49 @@ warnings.simplefilter('default')
 @use_rs_opt
 @geojson_type_feature_opt(True)
 @geojson_type_bbox_opt(False)
-@click.option('--bands/--mask', default=True,
-              help="Extract shapes from one of the dataset bands or from "
-                   "its nodata mask")
-@click.option('--bidx', type=int, default=1,
-              help="Index of the source band")
+@click.option('--band/--mask', default=True,
+              help="Choose to extract from a band (the default) or a mask.")
+@click.option('--bidx', 'bandidx', type=int, default=None,
+              help="Index of the band or mask that is the source of shapes.")
 @click.option('--sampling', type=int, default=1,
-              help="Inverse of the sampling fraction")
+              help="Inverse of the sampling fraction; "
+                   "a value of 10 decimates.")
 @click.option('--with-nodata/--without-nodata', default=False,
               help="Include or do not include (the default) nodata regions.")
+@click.option('--as-mask/--not-as-mask', default=False,
+              help="Interpret a band as a mask and output only one class of "
+                   "valid data shapes.")
 @click.pass_context
 def shapes(
         ctx, input, precision, indent, compact, projection, sequence,
-        use_rs, geojson_type, bands, bidx, sampling, with_nodata):
-    """Writes features of a dataset out as GeoJSON. It's intended for
-    use with single-band rasters and reads from the first band.
+        use_rs, geojson_type, band, bandidx, sampling, with_nodata, as_mask):
+    """Extracts shapes from one band or mask of a dataset and writes
+    them out as GeoJSON. Unless otherwise specified, the shapes will be
+    transformed to WGS 84 coordinates.
+
+    The default action of this command is to extract shapes from the
+    first band of the input dataset. The shapes are polygons bounding
+    contiguous regions (or features) of the same raster value. This
+    command performs poorly for int16 or float type datasets.
+
+    Bands other than the first can be specified using the `--bidx`
+    option:
+
+      $ rio shapes --bidx 3 tests/data/RGB.byte.tif
+
+    The valid data footprint of a dataset's i-th band can be extracted
+    by using the `--mask` and `--bidx` options:
+
+      $ rio shapes --mask --bidx 1 tests/data/RGB.byte.tif
+
+    Omitting the `--bidx` option results in a footprint extracted from
+    the conjunction of all band masks. This is generally smaller than
+    any individual band's footprint.
+
+    A dataset band may be analyzed as though it were a binary mask with
+    the `--as-mask` option:
+
+      $ rio shapes --as-mask --bidx 1 tests/data/RGB.byte.tif
     """
     # These import numpy, which we don't want to do unless it's needed.
     import numpy
@@ -64,6 +90,8 @@ def shapes(
         dump_kwds['separators'] = (',', ':')
     stdout = click.get_text_stream('stdout')
 
+    bidx = 1 if bandidx is None and band else bandidx
+
     # This is the generator for (feature, bbox) pairs.
     class Collection(object):
 
@@ -78,11 +106,11 @@ def shapes(
         def __call__(self):
             with rasterio.open(input) as src:
                 img = None
-                nodata_mask = None
-                if bands:
+                msk = None
+                if band:
                     if sampling == 1:
                         img = src.read(bidx, masked=False)
-                        transform = src.transform
+                        transform = src.affine
                     # Decimate the band.
                     else:
                         img = numpy.zeros(
@@ -90,16 +118,28 @@ def shapes(
                             dtype=src.dtypes[src.indexes.index(bidx)])
                         img = src.read(bidx, img, masked=False)
                         transform = src.affine * Affine.scale(float(sampling))
-                if not bands or not with_nodata:
+                    if as_mask:
+                        tmp = numpy.ones_like(img, 'uint8') * 255
+                        tmp[img == 0] = 0
+                        img = tmp
+                        msk = tmp
+                if not band or not with_nodata:
                     if sampling == 1:
-                        nodata_mask = src.read_masks(bidx)
-                        transform = src.transform
+                        msk = src.read_masks(bidx)
+                        if bidx is None:
+                            msk = numpy.logical_or.reduce(msk).astype('uint8')
+                        transform = src.affine
                     # Decimate the mask.
                     else:
-                        nodata_mask = numpy.zeros(
-                            (src.height//sampling, src.width//sampling),
-                            dtype=numpy.uint8)
-                        nodata_mask = src.read_masks(bidx, nodata_mask)
+                        msk_shape = src.height//sampling, src.width//sampling
+                        if bidx is None:
+                            msk = numpy.zeros(
+                                (src.count,) + msk_shape, 'uint8')
+                        else:
+                            msk = numpy.zeros(msk_shape, 'uint8')
+                        msk = src.read_masks(bidx, msk)
+                        if bidx is None:
+                            msk = numpy.logical_or.reduce(msk).astype('uint8')
                         transform = src.affine * Affine.scale(float(sampling))
 
                 bounds = src.bounds
@@ -116,10 +156,11 @@ def shapes(
 
                 kwargs = {'transform': transform}
                 # Default is to exclude nodata features.
-                if nodata_mask is not None:
-                    kwargs['mask'] = (nodata_mask==255)
+                if msk is not None:
+                    kwargs['mask'] = msk #(msk > 0)
                 if img is None:
-                    img = nodata_mask
+                    img = msk
+
                 for g, i in rasterio.features.shapes(img, **kwargs):
                     if projection == 'geographic':
                         g = rasterio.warp.transform_geom(
@@ -129,15 +170,18 @@ def shapes(
                     yield {
                         'type': 'Feature',
                         'id': str(i),
-                        'properties': {'val': i},
+                        'properties': {
+                            'val': i, 'filename': os.path.basename(src.name)
+                        },
                         'bbox': [min(xs), min(ys), max(xs), max(ys)],
-                        'geometry': g }
+                        'geometry': g
+                    }
 
     if not sequence:
         geojson_type = 'collection'
 
     try:
-        with rasterio.drivers(CPL_DEBUG=verbosity>2):
+        with rasterio.drivers(CPL_DEBUG=(verbosity > 2)):
             write_features(
                 stdout, Collection(), sequence=sequence,
                 geojson_type=geojson_type, use_rs=use_rs,
@@ -162,17 +206,19 @@ def shapes(
               help='Output dataset width, height in number of pixels.')
 @click.option('--res', multiple=True, type=float, default=None,
               help='Output dataset resolution in units of coordinate '
-              'reference system. Pixels assumed to be square if this option is '
-              'used once, otherwise use: '
+              'reference system. Pixels assumed to be square if this option '
+              'is used once, otherwise use: '
               '--res pixel_width --res pixel_height')
-@click.option('--src_crs', default='EPSG:4326',
+@click.option('--src_crs', default=None,
               help='Source coordinate reference system.  Limited to EPSG '
-              'codes for now.  Used as output coordinate system if output does '
-              'not exist or --like option is not used. Default: EPSG:4326')
+              'codes for now.  Used as output coordinate system if output '
+              'does not exist or --like option is not used. '
+              'Default: EPSG:4326')
 @click.option('--all_touched', is_flag=True, default=False)
 @click.option('--default_value', type=float, default=1, help='Default value '
               'for rasterized pixels')
-@click.option('--fill', type=float, default=0, help='Fill value for all pixels '
+@click.option('--fill', type=float, default=0,
+              help='Fill value for allpixels '
               'not overlapping features.  Will be evaluated as NoData pixels '
               'for output.  Default: 0')
 @click.option('--property', type=str, default=None, help='Property in '
@@ -197,24 +243,22 @@ def rasterize(
 
     If the output raster exists, rio-rasterize will rasterize feature values
     into all bands of that raster.  The GeoJSON is assumed to be in the same
-    coordinate reference system as the output.
+    coordinate reference system as the output unless --src_crs is provided.
 
     --default_value or property values when using --property must be using a
     data type valid for the data type of that raster.
 
 
     If a template raster is provided using the --like option, the affine
-    transform, coordinate reference system, and data type from that raster will
-    be used to create the output.
+    transform and data type from that raster will be used to create the output.
+    The GeoJSON is assumed to be in the same coordinate reference system unless
+    --src_crs is provided.
 
     --default_value or property values when using --property must be using a
     data type valid for the data type of that raster.
 
-    --driver, --bounds, --dimensions, --res, and --src_crs are ignored when
-    output exists or --like raster is provided
-
-    The GeoJSON must be within the bounds of the existing output or --like
-    raster, or an error will be raised.
+    --driver, --bounds, --dimensions, and --res are ignored when output exists
+    or --like raster is provided
 
 
     If the output does not exist and --like raster is not provided, the input
@@ -233,7 +277,7 @@ def rasterize(
     added in the future.
     """
 
-    import numpy as np
+    from rasterio._base import is_geographic_crs, is_same_crs
     from rasterio.features import rasterize
     from rasterio.features import bounds as calculate_bounds
 
@@ -242,6 +286,8 @@ def rasterize(
     files = list(files)
     output = files.pop()
     input = click.open_file(files.pop(0) if files else '-')
+    has_src_crs = src_crs is not None
+    src_crs = src_crs or 'EPSG:4326'
 
     # If values are actually meant to be integers, we need to cast them
     # as such or rasterize creates floating point outputs
@@ -278,10 +324,16 @@ def rasterize(
 
         if os.path.exists(output):
             with rasterio.open(output, 'r+') as out:
-                if disjoint_bounds(geojson_bounds, out.bounds):
-                    raise click.BadParameter('GeoJSON outside bounds of '
+                if has_src_crs and not is_same_crs(src_crs, out.crs):
+                    raise click.BadParameter('GeoJSON does not match crs of '
                                              'existing output raster',
-                                             param=input, param_hint='input')
+                                             param='input', param_hint='input')
+
+                if disjoint_bounds(geojson_bounds, out.bounds):
+                    click.echo("GeoJSON outside bounds of existing output "
+                               "raster. Are they in different coordinate "
+                               "reference systems?",
+                               err=True)
 
                 meta = out.meta.copy()
 
@@ -305,10 +357,17 @@ def rasterize(
         else:
             if like is not None:
                 template_ds = rasterio.open(like)
+
+                if has_src_crs and not is_same_crs(src_crs, template_ds.crs):
+                    raise click.BadParameter('GeoJSON does not match crs of '
+                                             '--like raster',
+                                             param='input', param_hint='input')
+
                 if disjoint_bounds(geojson_bounds, template_ds.bounds):
-                    raise click.BadParameter('GeoJSON outside bounds of '
-                                             '--like raster', param=input,
-                                             param_hint='input')
+                    click.echo("GeoJSON outside bounds of --like raster. "
+                               "Are they in different coordinate reference "
+                               "systems?",
+                               err=True)
 
                 kwargs = template_ds.meta.copy()
                 kwargs['count'] = 1
@@ -317,11 +376,12 @@ def rasterize(
             else:
                 bounds = bounds or geojson_bounds
 
-                if src_crs == 'EPSG:4326':
+                if is_geographic_crs(src_crs):
                     if (bounds[0] < -180 or bounds[2] > 180 or
-                        bounds[1] < -80 or bounds[3] > 80):
+                            bounds[1] < -80 or bounds[3] > 80):
                         raise click.BadParameter(
-                            'Bounds are beyond the valid extent for EPSG:4326.',
+                            "Bounds are beyond the valid extent for "
+                            "EPSG:4326.",
                             ctx, param=bounds, param_hint='--bounds')
 
                 if dimensions:
